<?php
/**
 * @copyright	Copyright (c) 2014 Skyline Technology Ltd (http://extstore.com). All rights reserved.
 * @license		http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */

// no direct access
defined('_JEXEC') or die;

/**
 * Moods Model.
 *
 * @package		Joomla.Site
 * @subpakage	ExtStore.MoodPoll
 */
class MoodPollModelStatistic extends JModelList {

	/**
	 * Constructor.
	 *
	 * @param   array  $config  An optional associative array of configuration settings.
	 *
	 * @see     JController
	 */
	public function __construct($config = array()) {
		if (empty($config['filter_fields'])) {
			$config['filter_fields'] = array(
				'id', 'a.id',
				'title', 'a.title',
				'image', 'a.image',
				'ordering', 'a.ordering'
			);
		}

		parent::__construct($config);

		$this->setState('list.start', 0);
		$this->setState('list.limit', 0);
		$this->setState('list.order', 'ordering');
		$this->setState('list.direction', 'ASC');
	}

	/**
	 * Populate state.
	 */
	public function populateState() {
		$date		= JFactory::getDate();
		$start_date	= $this->getUserStateFromRequest('filter.start_date', 'start_date', $date->setTime(0, 0)->toSql());
		$end_date	= $this->getUserStateFromRequest('filter.end_date', 'end_date', $date->setTime(23, 59, 59)->toSql());

		$this->setState('filter.start_date', $start_date);
		$this->setState('filter.end_date', $end_date);
	}

	/**
	 * Method to get a store id based on model configuration state.
	 *
	 * This is necessary because the model is used by the component and
	 * different modules that might need different sets of data or different
	 * ordering requirements.
	 *
	 * @param   string  $id  A prefix for the store id.
	 *
	 * @return  string  A store id.
	 */
	protected function getStoreId($id = '') {
		// compile the store id
		$id	.= ':' . serialize($this->getState('filter.state'));
		$id	.= ':' . $this->getState('filter.access');
		$id	.= ':' . $this->getState('filter.start_date');
		$id	.= ':' . $this->getState('filter.end_date');
		$id	.= ':' . serialize($this->getState('filter.mood_id'));
		$id	.= ':' . $this->getState('filter.mood_id.include');

		return parent::getStoreId($id);
	}

	/**
	 * Get the master query for retrieving a list of articles subject to the model state.
	 *
	 * @return  JDatabaseQuery
	 */
	protected function getListQuery() {
		// Get the current user for authorisation checks
		$user	= JFactory::getUser();

		// Create a new query object.
		$db = $this->getDbo();
		$query = $db->getQuery(true);

		// Select the required fields from the table.
		$query->select(
			$this->getState('list.select',
				'a.id, a.title, a.image, a.message'
			)
		);

		$query->from('#__moodpoll_moods AS a');

		// Join over the logs table
		$conditions	= array('l.mood_id = a.id');

		if ($start_date = $this->getState('filter.start_date')) {
			$conditions[]	= 'l.created >= ' . $db->q($start_date);
		}

		if ($end_date = $this->getState('filter.end_date')) {
			$conditions[]	= 'l.created <= ' . $db->q($end_date);
		}


		$query->select('COUNT(l.id) AS votes');
		$query->group('a.id');
		$query->join('left', '#__moodpoll_logs AS l ON ' . implode(' AND ', $conditions));

		// Filter by access level.
		if ($access = $this->getState('filter.access')) {
			$groups = implode(',', $user->getAuthorisedViewLevels());

			$query->where('a.access IN (' . $groups . ')');
		}

		// Filter by published state
		$state = $this->getState('filter.state');

		if (is_numeric($state)) {
			$query->where('a.state = ' . (int) $state);
		} else if (is_array($state)) {
			JArrayHelper::toInteger($state);
			$state = implode(',', $state);

			$query->where('a.state IN (' . $state . ')');
		}

		// Filter by a single or group of articles.
		$moodId = $this->getState('filter.mood_id');

		if (is_numeric($moodId)) {
			$type = $this->getState('filter.mood_id.include', true) ? '= ' : '<> ';
			$query->where('a.id ' . $type . (int) $moodId);
		} else if (is_array($moodId)) {
			JArrayHelper::toInteger($moodId);
			$moodId = implode(',', $moodId);
			$type = $this->getState('filter.mood_id.include', true) ? 'IN' : 'NOT IN';
			$query->where('a.id ' . $type . ' (' . $moodId . ')');
		}

		return $query;
	}

	/**
	 * Method to get an number of votes.
	 *
	 * @return  int  Number of votes.
	 */
	public function getTotalVotes() {
		// Get a storage key.
		$store = $this->getStoreId('getTotalVotes');

		// Try to load the data from internal storage.
		if (isset($this->cache[$store])) {
			return $this->cache[$store];
		}

		$items	= $this->getItems();
		$total	= 0;

		foreach ($items as $item) {
			$total	+=	$item->votes;
		}


		// Add the items to the internal cache.
		$this->cache[$store] = $total;

		return $this->cache[$store];
	}
}
