<?php
/**
 * @copyright	Copyright (c) 2014 Skyline Technology Ltd (http://extstore.com). All rights reserved.
 * @license		http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */

// no direct access
defined('_JEXEC') or die;

/**
 * Moods Model.
 *
 * @package		Joomla.Site
 * @subpakage	ExtStore.MoodPoll
 */
class MoodPollModelMoods extends JModelList {

	/**
	 * Constructor.
	 *
	 * @param   array  $config  An optional associative array of configuration settings.
	 *
	 * @see     JController
	 */
	public function __construct($config = array()) {
		if (empty($config['filter_fields'])) {
			$config['filter_fields'] = array(
				'id', 'a.id',
				'title', 'a.title',
				'image', 'a.image',
				'ordering', 'a.ordering'
			);
		}

		parent::__construct($config);

		// calculate start date and end date
		$date		= JFactory::getDate();
		$start_date	= $date->setTime(0, 0)->toSql();
		$end_date	= $date->setTime(23, 59, 59)->toSql();

		$this->setState('filter.start_date', $start_date);
		$this->setState('filter.end_date', $end_date);

		//
		$this->setState('filter.state', 1);
		$this->setState('filter.access', true);

		$this->setState('list.start', 0);
		$this->setState('list.limit', 0);
		$this->setState('list.order', 'ordering');
		$this->setState('list.direction', 'ASC');
	}

	/**
	 * Method to get a store id based on model configuration state.
	 *
	 * This is necessary because the model is used by the component and
	 * different modules that might need different sets of data or different
	 * ordering requirements.
	 *
	 * @param   string  $id  A prefix for the store id.
	 *
	 * @return  string  A store id.
	 */
	protected function getStoreId($id = '') {
		// compile the store id
		$id	.= ':' . serialize($this->getState('filter.state'));
		$id	.= ':' . $this->getState('filter.access');
		$id	.= ':' . $this->getState('filter.start_date');
		$id	.= ':' . $this->getState('filter.end_date');
		$id	.= ':' . serialize($this->getState('filter.mood_id'));
		$id	.= ':' . $this->getState('filter.mood_id.include');

		return parent::getStoreId($id);
	}

	/**
	 * Get the master query for retrieving a list of articles subject to the model state.
	 *
	 * @return  JDatabaseQuery
	 */
	protected function getListQuery() {
		// Get the current user for authorisation checks
		$user	= JFactory::getUser();

		// Create a new query object.
		$db = $this->getDbo();
		$query = $db->getQuery(true);

		// Select the required fields from the table.
		$query->select(
			$this->getState('list.select',
				'a.id, a.title, a.image, a.message'
			)
		);

		$query->from('#__moodpoll_moods AS a');

		// Join over the logs table
		$conditions	= array('l.mood_id = a.id');

		if ($start_date = $this->getState('filter.start_date')) {
			$conditions[]	= 'l.created >= ' . $db->q($start_date);
		}

		if ($end_date = $this->getState('filter.end_date')) {
			$conditions[]	= 'l.created <= ' . $db->q($end_date);
		}


		$query->select('COUNT(l.id) AS votes');
		$query->group('a.id');
		$query->join('left', '#__moodpoll_logs AS l ON ' . implode(' AND ', $conditions));

		// Filter by access level.
		if ($access = $this->getState('filter.access')) {
			$groups = implode(',', $user->getAuthorisedViewLevels());

			$query->where('a.access IN (' . $groups . ')');
		}

		// Filter by published state
		$state = $this->getState('filter.state');

		if (is_numeric($state)) {
			$query->where('a.state = ' . (int) $state);
		} else if (is_array($state)) {
			JArrayHelper::toInteger($state);
			$state = implode(',', $state);

			$query->where('a.state IN (' . $state . ')');
		}

		// Filter by a single or group of articles.
		$moodId = $this->getState('filter.mood_id');

		if (is_numeric($moodId)) {
			$type = $this->getState('filter.mood_id.include', true) ? '= ' : '<> ';
			$query->where('a.id ' . $type . (int) $moodId);
		} else if (is_array($moodId)) {
			JArrayHelper::toInteger($moodId);
			$moodId = implode(',', $moodId);
			$type = $this->getState('filter.mood_id.include', true) ? 'IN' : 'NOT IN';
			$query->where('a.id ' . $type . ' (' . $moodId . ')');
		}

		return $query;
	}

	/**
	 * Method to get an number of votes.
	 *
	 * @return  int  Number of votes.
	 */
	public function getTotalVotes() {
		// Get a storage key.
		$store = $this->getStoreId('getTotalVotes');

		// Try to load the data from internal storage.
		if (isset($this->cache[$store])) {
			return $this->cache[$store];
		}

		$items	= $this->getItems();
		$total	= 0;

		foreach ($items as $item) {
			$total	+=	$item->votes;
		}


		// Add the items to the internal cache.
		$this->cache[$store] = $total;

		return $this->cache[$store];
	}

	/**
	 * Check vote permission.
	 *
	 * @return	int	id of mood that user voted. 0 if user doesn't have permission, -1 if user hasn't voted.
	 */
	public function checkPermission($moodId = 0) {
		// Get a storage key.
		$store	= $this->getStoreId('checkPermission:' . $moodId);

		// Try to load the data from internal storage.
		if (isset($this->cache[$store])) {
			return $this->cache[$store];
		}

		$this->cache[$store]	= $this->_checkPermission($moodId);

		return $this->cache[$store];
	}

	protected function _checkPermission($moodId = 0) {
		$user	= JFactory::getUser();
		$db		= $this->getDbo();

		if (!$user->authorise('vote', 'com_moodpoll')) {
			return 0;
		}

		if ($moodId) {
			$query	= $db->getQuery(true);

			$query->select('id')
				->from('#__moodpoll_moods AS a')
				->where('id = ' . (int) $moodId)
			;

			// Filter by access level.
			if ($access = $this->getState('filter.access')) {
				$groups = implode(',', $user->getAuthorisedViewLevels());

				$query->where('a.access IN (' . $groups . ')');
			}

			// Filter by published state
			$state = $this->getState('filter.state');

			if (is_numeric($state)) {
				$query->where('a.state = ' . (int) $state);
			} else if (is_array($state)) {
				JArrayHelper::toInteger($state);
				$state = implode(',', $state);

				$query->where('a.state IN (' . $state . ')');
			}

			// Filter by a single or group of moods.
			$moodIds = $this->getState('filter.mood_id');

			if (is_numeric($moodIds)) {
				$type = $this->getState('filter.mood_id.include', true) ? '= ' : '<> ';
				$query->where('a.id ' . $type . (int) $moodIds);
			} else if (is_array($moodIds)) {
				JArrayHelper::toInteger($moodIds);
				$moodIds = implode(',', $moodIds);
				$type = $this->getState('filter.mood_id.include', true) ? 'IN' : 'NOT IN';
				$query->where('a.id ' . $type . ' (' . $moodIds . ')');
			}

			$db->setQuery($query);

			if (!$db->loadResult()) {
				return 0;
			}
		}

		if ($user->id) {
			// Check permission by user_id
			$query	= $db->getQuery(true);

			$query->select('mood_id')
				->from('#__moodpoll_logs AS a')
				->where('user_id = ' . (int) $user->id)
			;

			if ($start_date = $this->getState('filter.start_date')) {
				$query->where('a.created >= ' . $db->q($start_date));
			}

			if ($end_date = $this->getState('filter.end_date')) {
				$query->where('a.created <= ' . $db->q($end_date));
			}

			$db->setQuery($query);
			$moodId	= $db->loadResult();

			if ($moodId) {
				return $moodId;
			}
		}

		//
		$app		= JFactory::getApplication();
		$config		= JComponentHelper::getParams('com_moodpoll');
		$check_type	= $config->get('check_type');

		if ($check_type == 0) {
			// check by cookie
			$moodId	= $app->input->cookie->get('moodId');

			if ($moodId) {
				return $moodId;
			}
		} else {
			// check by ip
			$ip		= $app->input->server->get('REMOTE_ADDR');
			$query	= $db->getQuery(true);

			$query->select('mood_id')
				->from('#__moodpoll_logs AS a')
				->where('ip = ' . $db->q($ip))
			;

			if ($start_date = $this->getState('filter.start_date')) {
				$query->where('a.created >= ' . $db->q($start_date));
			}

			if ($end_date = $this->getState('filter.end_date')) {
				$query->where('a.created <= ' . $db->q($end_date));
			}

			$db->setQuery($query);
			$moodId	= $db->loadResult();

			if ($moodId) {
				return $moodId;
			}
		}

		return -1;
	}

	/**
	 * Get chosen mood.
	 */
	public function getChosenMood() {
		if ($chosenMoodId = $this->getState('item.chosen')) {
			$checkPerm		= $chosenMoodId;
		} else {
			$checkPerm		= $this->checkPermission();
		}

		$items			= $this->getItems();
		$chosenMood		= null;

		if ($checkPerm > 0) {
			foreach ($items as $item) {
				if ($item->id == $checkPerm) {
					$chosenMood	= $item;
					break;
				}
			}
		}

		return $chosenMood;
	}

	/**
	 * Vote a mood.
	 */
	public function vote($moodId) {
		$app		= JFactory::getApplication();
		$user		= JFactory::getUser();
		$date		= JFactory::getDate();
		$data		= array();

		$data['id']			= 0;
		$data['mood_id']	= $moodId;
		$data['user_id']	= (int) $user->id;
		$data['ip']			= $app->input->server->get('REMOTE_ADDR');
		$data['created']	= $date->toSql();

		// set cookie
		$expire		= null;

		if ($end_date = $this->getState('filter.end_date')) {
			$expire	= JFactory::getDate($end_date)->toUnix();
		}

		setcookie('moodId', $moodId, $expire);

		JTable::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_moodpoll/tables');
		$table		= JTable::getInstance('Log', 'MoodPollTable');
		$table->bind($data);

		if ($table->check()) {
			if ($table->store()) {
				$this->setState('item.chosen', $moodId);

				return true;
			}
		}

		return false;
	}
}
